@@ -43,7 +43,11 @@
 #include <cstdlib>
 #include <cstring>
 
-#ifndef	WIN32
+#ifdef	WIN32
+#ifndef	_WIN32_WCE
+#include <sys/timeb.h>
+#endif
+#else
 
 #ifdef  HAVE_SYS_PARAM_H
 #include <sys/param.h>
@@ -82,6 +86,7 @@ using namespace std;
 #endif
 
 #ifdef	WIN32
+#ifdef	_WIN32_WCE
 int gettimeofday(struct timeval *tv_,  void *tz_)
 {
         // We could use _ftime(), but it is not available on WinCE.
@@ -92,6 +97,21 @@ int gettimeofday(struct timeval *tv_,  void *tz_)
         tv_->tv_usec = ms * 1000;
         return 0;
 }
+#else
+int gettimeofday(struct timeval *tv_, void *tz_)
+{
+#if defined(_MSC_VER) && _MSC_VER >= 1300
+	struct __timeb64 tb;
+	_ftime64(&tb);
+#else
+	struct _timeb tb;
+	_ftime(&tb);
+#endif
+	tv_->tv_sec = tb.time;
+	tv_->tv_usec = tb.millitm * 1000;
+	return 0;
+}	
+#endif
 #endif
 
 #ifndef	WIN32
