@@ -44,6 +44,10 @@
 #include <iostream>
 #include <cerrno>
 
+#ifdef HAVE_GCC_CXX_BITS_ATOMIC
+using namespace __gnu_cxx;
+#endif
+
 #ifdef	CCXX_NAMESPACES
 namespace ost {
 #endif
@@ -411,7 +415,8 @@ int operator--(MutexCounter &mc)
 #ifdef	HAVE_ATOMIC
 AtomicCounter::AtomicCounter()
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	counter = 0;
 #else
 	atomic.counter = 0;
@@ -420,7 +425,8 @@ AtomicCounter::AtomicCounter()
 
 AtomicCounter::AtomicCounter(int value)
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	counter = 0;
 #else
 	atomic.counter = value;
@@ -433,6 +439,8 @@ int AtomicCounter::operator++(void)
 {
 #ifdef	HAVE_ATOMIC_AIX
 	return fetch_and_add((atomic_p)&counter, 1);
+#elif   defined(HAVE_GCC_BITS_ATOMIC) || defined(HAVE_GCC_CXX_BITS_ATOMIC)
+	return __exchange_and_add(&counter, 1);
 #else
 	atomic_inc(&atomic);
 	return atomic_read(&atomic);
@@ -443,6 +451,8 @@ int AtomicCounter::operator--(void)
 {
 #ifdef	HAVE_ATOMIC_AIX
 	return fetch_and_add((atomic_p)&counter, -1);
+#elif   defined(HAVE_GCC_BITS_ATOMIC) || defined(HAVE_GCC_CXX_BITS_ATOMIC)
+	return __exchange_and_add(&counter, -1);
 #else
 	int chk = atomic_dec_and_test(&atomic);
 	if(chk)
@@ -458,6 +468,8 @@ int AtomicCounter::operator+=(int change)
 {
 #ifdef	HAVE_ATOMIC_AIX
 	return fetch_and_add((atomic_p)&counter, change);
+#elif   defined(HAVE_GCC_BITS_ATOMIC) || defined(HAVE_GCC_CXX_BITS_ATOMIC)
+	return __exchange_and_add(&counter, change);
 #else
 	atomic_add(change, &atomic);
 	return atomic_read(&atomic);
@@ -468,6 +480,8 @@ int AtomicCounter::operator-=(int change)
 {
 #ifdef	HAVE_ATOMIC_AIX
 	return fetch_and_add((atomic_p)&counter, -change);
+#elif   defined(HAVE_GCC_BITS_ATOMIC) || defined(HAVE_GCC_CXX_BITS_ATOMIC)
+	return __exchange_and_add(&counter, -change);
 #else
 	atomic_sub(change, &atomic);
 	return atomic_read(&atomic);
@@ -476,7 +490,8 @@ int AtomicCounter::operator-=(int change)
 
 int AtomicCounter::operator+(int change)
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	return counter + change;
 #else
 	return atomic_read(&atomic) + change;
@@ -485,7 +500,8 @@ int AtomicCounter::operator+(int change)
 
 int AtomicCounter::operator-(int change)
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	return counter - change;
 #else
 	return atomic_read(&atomic) - change;
@@ -494,7 +510,8 @@ int AtomicCounter::operator-(int change)
 
 int AtomicCounter::operator=(int value)
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	return counter = value;
 #else
 	atomic_set(&atomic, value);
@@ -504,7 +521,8 @@ int AtomicCounter::operator=(int value)
 
 bool AtomicCounter::operator!(void)
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	int value = counter;
 #else
 	int value = atomic_read(&atomic);
@@ -516,7 +534,8 @@ bool AtomicCounter::operator!(void)
 
 AtomicCounter::operator int()
 {
-#ifdef	HAVE_ATOMIC_AIX
+#if     defined(HAVE_ATOMIC_AIX) || defined(HAVE_GCC_BITS_ATOMIC) \
+    || defined(HAVE_GCC_CXX_BITS_ATOMIC)
 	return counter;
 #else
 	return atomic_read(&atomic);
