@@ -1483,12 +1483,18 @@ void ttystream::open(const char *name)
 }
 
 TTYSession::TTYSession(const char *filename, int pri, int stack) :
-TTYStream(filename), Thread(pri, stack)
+Thread(pri, stack), TTYStream(filename)
 {
 	setError(false);
 }
 
 
+TTYSession::~TTYSession()
+{
+	terminate();
+	endSerial();
+}
+
 #ifndef	WIN32
 //	Not supporting this right now........
 //
