@@ -1602,13 +1602,14 @@ Socket::Error UDPSocket::join(const IPV4Multicast &ia,int InterfaceIndex)
         if(!flags.multicast)
           return error(errMulticastDisabled);
 
+	memset(&group, 0, sizeof(group));
         getsockname(so, (struct sockaddr *)&myaddr, &len);  
         group.imr_multiaddr.s_addr = ia.getAddress().s_addr;
         group.imr_interface.s_addr = INADDR_ANY;
-		int retval = setsockopt(so, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&group, sizeof(group));
-		return errSuccess;
+	int retval = setsockopt(so, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&group, sizeof(group));
+	return errSuccess;
 
-#elif defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__) && !defined(_OSF_SOURCE) && !defined(__hpux)
+#elif defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__) && !defined(_OSF_SOURCE) && !defined(__hpux) && !defined(__GNU__)
         
 		struct ip_mreqn      group;
         struct sockaddr_in   myaddr;
@@ -1618,11 +1619,27 @@ Socket::Error UDPSocket::join(const IPV4Multicast &ia,int InterfaceIndex)
           return error(errMulticastDisabled);
 
         getsockname(so, (struct sockaddr *)&myaddr, &len);
+	memset(&group, 0, sizeof(group));
         memcpy(&group.imr_address, &myaddr.sin_addr, sizeof(&myaddr.sin_addr));
         group.imr_multiaddr = getaddress(ia);
         group.imr_ifindex   = InterfaceIndex;
         setsockopt(so, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&group, sizeof(group));
         return errSuccess;
+#elif defined(IP_ADD_MEMBERSHIP)
+	// if no by index, use INADDR_ANY
+	struct ip_mreq group;
+	struct sockaddr_in myaddr;
+	socklen_t len = sizeof(myaddr);
+
+	if(!flags.multicast)
+		return error(errMulticastDisabled);
+
+	getsockname(so, (struct sockaddr *)&myaddr, &len);
+	memset(&group, sizeof(group), 0);
+	group.imr_multiaddr.s_addr = ia.getAddress().s_addr;
+	group.imr_interface.s_addr = INADDR_ANY;
+	setsockopt(so, IPPROTO_IP, IP_ADD_MEMBERSHIP, (char *)&group, sizeof(group));
+	return errSuccess;
 #else
         return error(errServiceUnavailable);
 #endif
@@ -1632,7 +1649,7 @@ Socket::Error UDPSocket::join(const IPV4Multicast &ia,int InterfaceIndex)
 Socket::Error UDPSocket::getInterfaceIndex(const char *DeviceName,int& InterfaceIndex)
 {
 #ifndef WIN32
-#if defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__) && !defined(_OSF_SOURCE) && !defined(__hpux)
+#if defined(IP_ADD_MEMBERSHIP) && defined(SIOCGIFINDEX) && !defined(__FreeBSD__) && !defined(_OSF_SOURCE) && !defined(__hpux) && !defined(__GNU__)
 
   struct ip_mreqn  mreqn;
   struct ifreq     m_ifreq;
@@ -1651,7 +1668,7 @@ Socket::Error UDPSocket::getInterfaceIndex(const char *DeviceName,int& Interface
   if (ioctl (so, SIOCGIFINDEX, &m_ifreq))
     return error(errServiceUnavailable);
 
-#if defined(__FreeBSD__)
+#if defined(__FreeBSD__) || defined(__GNU__)
   InterfaceIndex = m_ifreq.ifr_ifru.ifru_index;
 #else
   InterfaceIndex = m_ifreq.ifr_ifindex;
